--[[
	Copyright (C) Udorn (Blackhand)
--]]

petbm = petbm or {}

petbm.PetUtil = {}

local DEAL_DAMAGE_PLUS = 1
local DEAL_DAMAGE_MINUS = 2
local RECEIVE_DAMAGE_PLUS = 3
local RECEIVE_DAMAGE_MINUS = 4

local HUMANOID = petbm.PetConstants.HUMANOID
local DRAGON = petbm.PetConstants.DRAGON
local FLYING = petbm.PetConstants.FLYING
local UNDEAD = petbm.PetConstants.UNDEAD
local CRITTER = petbm.PetConstants.CRITTER
local MAGICAL = petbm.PetConstants.MAGICAL
local ELEMENTAL = petbm.PetConstants.ELEMENTAL
local BEAST = petbm.PetConstants.BEAST
local WATER = petbm.PetConstants.WATER
local MECHANICAL = petbm.PetConstants.MECHANICAL

local MODIFIER_TABLE = {
	{DRAGON, BEAST, UNDEAD, CRITTER}, -- HUMANOID
	{MAGICAL, UNDEAD, HUMANOID, FLYING}, -- DRAGON
	{WATER, DRAGON, MAGICAL, BEAST}, -- FLYING
	{HUMANOID, WATER, CRITTER, DRAGON}, -- UNDEAD
	{UNDEAD, HUMANOID, BEAST, nil}, -- CRITTER
	{FLYING, MECHANICAL, DRAGON, WATER}, -- MAGICAL
	{MECHANICAL, nil, WATER, MECHANICAL}, -- ELEMENTAL
	{CRITTER, FLYING, MECHANICAL, HUMANOID}, -- BEAST
	{ELEMENTAL, MAGICAL, FLYING, UNDEAD}, -- WATER
	{BEAST, ELEMENTAL, ELEMENTAL, MAGICAL} -- MECHANICAL
}

-- Returns dealDmgPlus, dealDmgMinus, receiveDmgPlus, receiveDmgMinus for the given pet type.
-- The modifiers will contain the target pet type. 
local function GetDamageModifiers(petType)
	return unpack(MODIFIER_TABLE[petType])
end

-- Read the current state of the pet journal. Should be called at startup and after 
-- having received PET_JOURNAL_LIST_UPDATE.
local function InitPetJournal()

	petbm.PetUtil.rarityCache = wipe(petbm.PetUtil.rarityCache or {})
	local rarityCache = petbm.PetUtil.rarityCache
	 
	-- we have to reset filter, therefore we are caching the results
	C_PetJournal.SetSearchFilter("")
	local numPets = C_PetJournal.GetNumPets(false)
	for i = 1,numPets do
		local petId, speciesId, owned = C_PetJournal.GetPetInfoByIndex(i, false)
		if (owned) then
			local health, maxHealth, attack, speed, rarity = C_PetJournal.GetPetStats(petId)
			local r = rarityCache[speciesId]
			if (r == nil or r < rarity) then
				rarityCache[speciesId] = rarity
			end
		end
	end
end

-- Returns the highest rarity for the given pet, or nil if not found.
local function GetPetRarityFromJournal(speciesId)
	return petbm.PetUtil.rarityCache[speciesId]
end

petbm.PetUtil.GetDamageModifiers = GetDamageModifiers
petbm.PetUtil.InitPetJournal = InitPetJournal
petbm.PetUtil.GetPetRarityFromJournal = GetPetRarityFromJournal
